package de.hhu.phil.smt;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.Map;
import java.util.Scanner;

import de.hhu.phil.smt.lm.LanguageModel;

/*
 * Author: kaeshammer@uni-duesseldorf.de
 * 
 * Collection of static helper methods that might be useful.
 */

public class SMT {
	
	/*
	 *  start-of-sentence token
	 */
	public static final String START_TOK = "<s>"; 
	
	/*
	 *  end-of-sentence token
	 */
	public static final String END_TOK = "</s>";
	
	/*
	 *  unknown token
	 */
	public static final String UNK = "<UNK>";
	
	/*
	 * Null-alignment token
	 */
	public static final String NULL_ALIGN = "<NULL>";

	/*
	 * Tokenize str according to whitespace.
	 */
	public static String[] tokenizeWS(String str) {
		return str.split("\\s+");
	}
	
	/*
	 * Add null-alignment token to the beginning of the line
	 */
	public static String addNullAlign(String line) {
		return NULL_ALIGN + " " + line;
	}

	/*
	 * Add start-of-sentence and end-of-sentence token
	 */
	public static String addStartEnd(String line) {
		return START_TOK + " " + line + " " + END_TOK;
	}
	
	/*
	 * Add start-of-sentence twice and end-of-sentence token
	 */
	public static String addStartStartEnd(String line) {
		return START_TOK + " " + START_TOK + " " + line + " " + END_TOK;
	}
	
	/*
	 * Print the map with each entry on a line
	 */
	public static <K, V> void pprintHashMap(HashMap<K,V> map) {
		for (Map.Entry<K, V> p : map.entrySet()) {
			System.out.println(p);
		}
	}

	/*
	 * Perplexity of the language model on the test data dataFile
	 */
	public static double perplexity(String dataFile, LanguageModel lm) throws FileNotFoundException {
		Scanner input = new Scanner(new File(dataFile));
		String line = "";
		int n = 0;
		double s = 0;
		while (input.hasNextLine()) {
			line = input.nextLine();
			String[] toks = SMT.tokenizeWS(line);
			n += toks.length+1; // plus one for EOS 
			s += lm.logScore(line);
		}
		input.close();
		return Math.exp(s/(-n));
	}

}
