package de.hhu.phil.smt.ibm;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Scanner;
import java.util.Map.Entry;

import de.hhu.phil.smt.SMT;

/*
 * Author: kaeshammer@phil.uni-duesseldorf.de
 * 
 * Class for IBM Model 1
 */
public class IBMModel1 {
	
	/*
	 * Number of EM iterations
	 */
	private int numIter;
	
	/*
	 * Data structures for 
	 * a. the German (source) data
	 * b. the English (target) data
	 * c. the English (target) vocabulary 
	 * d. the alignment (English to German).
	 * 
	 * a., b. and d. contain one entry for each sentence pair in 
	 * the training data. The order of the training data is preserved.
	 */
	private ArrayList<String[]> dataD = new ArrayList<String[]>();
	private ArrayList<String[]> dataE = new ArrayList<String[]>();
	private HashSet<String> vocabularyE = new HashSet<String>();
	private ArrayList<int[]> alignment = new ArrayList<int[]>();
	
	/*
	 * Data structure for the lexical translation probabilities P(e|d):
	 * (1) German word -> English word -> translation probability
	 */
	// TODO (1)
	
	/*
	 * Data structures for the counts (necessary during training).
	 * (2)  German word -> English word -> count
	 * (3)  German word -> count
	 */
	// TODO (2)
	// TODO (3)

	
	/**
	 * Constructor reads the data files, runs EM for IBM1 to train the 
	 * alignment/the lexical translation probabilities, and establishes 
	 * the most probable alignment for each sentence pair based on the training.
	 * 
	 * @param fileD German (source) file, one sentence per line, tokenized by whitespace
	 * @param fileE English (target) file, one sentence per line aligned to fileD, 
	 * tokenized by whitespace
	 * @param iter number of iterations of EM
	 */
	public IBMModel1(String fileD, String fileE, int iter) throws FileNotFoundException {		
		numIter = iter;
		
		readData(fileD, fileE); // already implemented
		run(); // your job
		findMostProbableAlignments(); // your job
	}
	
	
	/*
	 * Read the data and put it into the data structures
	 */
	private void readData(String fileD, String fileE) throws FileNotFoundException {
		Scanner inputD = new Scanner(new File(fileD));
		Scanner inputE = new Scanner(new File(fileE));
		String german = "";
		String english = "";
		int c = 0;
		while (inputD.hasNextLine() && inputE.hasNextLine()) {
			german = inputD.nextLine();
			english = inputE.nextLine();
			c++;
			
			// tokenize; add the NULL_ALIGN special token to the start of
			// the German sentence
			String[] tokensD = SMT.tokenizeWS(SMT.addNullAlign(german));
			String[] tokensE = SMT.tokenizeWS(english);
			
			dataE.add(tokensE);
			dataD.add(tokensD);
			
			// English vocabulary
			for (String e : tokensE) {
				vocabularyE.add(e);
			}
		}
		if (inputD.hasNextLine() || inputE.hasNextLine()) {
			System.out.println("Warning: One input file is longer than the other.");
			System.out.println("Only considering the first " + c + " lines of both files.");
		}
		System.out.println("Number of sentence pairs: " + c);
		inputD.close();
		inputE.close();
	}

	/*
	 * Run EM for IBM Model 1
	 */
	private void run() {
		// TODO
	}

	
	/*
	 * After the EM-Training, find the most probable alignment
	 * for each sentence pair and store it in alignment.
	 */
	private void findMostProbableAlignments() {
		// TODO
	}
	
	

	/*
	 * Write the lexical translation table P(e|d) to a file.
	 * The format is one German word d with all possible translations e_i and their 
	 * probabilities p_i per line:
	 * "d \t e_1=p_1 \t e_2=p_2 ... e_n=p_n" (without the spaces)
	 * If a combination of d and e does not appear in this table, then P(e|d)=0.
	 * 
	 * @param file output file
	 */
	public void writeTransProbTable2File(String file) throws IOException {
        BufferedWriter out = new BufferedWriter(new FileWriter(file));
        
        // TODO
        
        // You can use 
        // out.write("blah");
        // to write blah to the file and 
        // out.newLine();
        // to add a linebreak to the file.

        out.close();		
	}

	/*
	 * Write the most probable alignment of each sentence pair in the training
	 * data to a file.
	 * 
	 * The format is the following: Each line represents the alignment of one
	 * sentence pair - the nth line in the file thus corresponds to the nth 
	 * sentence pair. Each line has the following form:
	 * "j_1-i_1 j_2-i_2 j_3-i_3 ... "
	 * j_x are indices into the German (source) sentence, i_y are indices into
	 * the English (target) sentence. Indexing starts with 0. j_x-i_y means that
	 * the j_xth German word is aligned to the i_yth English word. 
	 * If an index into English does not occur, it means that the corresponding 
	 * English word is not aligned (i.e. that it is aligned to NULL).
	 * 
	 * @param file output file
	 */
	public void writeMostProbableAlignments2File(String file) throws IOException {
		BufferedWriter out = new BufferedWriter(new FileWriter(file));
		for (int n = 0; n < alignment.size(); n++) {
			int[] align = alignment.get(n);
			
			// TODO
			
			// You can use 
	        // out.write("blah");
	        // to write blah to the file.
			
			out.newLine();
		}
        out.close();
	}

}
