package de.hhu.phil.smt.lm;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;

import de.hhu.phil.smt.SMT;


/*
 * Author: kaeshammer@phil.uni-duesseldorf.de
 * 
 * Class für Uebung 2 of the course Statistische Maschinelle Übersetzung 2013 
 * at HHU
 */
public class Uebung2 {
	
	/*
	 * @Param dataFile file with the training data
	 * @Param threshold frequency threshold
	 * Determines the vocabulary given the training data. 
	 * Only words that occur more frequently than the threshold are
	 * part of the vocabulary.
	 */
	private static Set<String> determineVocabulary(String dataFile, int threshold) throws FileNotFoundException {
		// data structure to count word frequencies
		HashMap<String, Integer> counts = new HashMap<String, Integer>();
		// read in the training data file line by line
		Scanner input = new Scanner(new File(dataFile));
		String line = "";
		while (input.hasNextLine()) {
			line = input.nextLine();
			// tokenize the sentence
			String[] tokens = SMT.tokenizeWS(line);
			// fill the data structure
			for (String tok : tokens) {
				Integer c = counts.get(tok);
				if (c == null) {
					c = 0;
				}
				counts.put(tok, c+1);
			}
		}
		input.close();
		
		// add words which we have seen more often than threshold to the vocabulary
		Set<String> vocabulary = new HashSet<String>();
		for (Map.Entry<String, Integer> entry : counts.entrySet()) {
			if (entry.getValue() > threshold) {
				vocabulary.add(entry.getKey());
			}
		}
		
		vocabulary.add(SMT.START_TOK);
		vocabulary.add(SMT.END_TOK);
		vocabulary.add(SMT.UNK);
		
		return vocabulary;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		String trainData = "data/train.de.tok.50.h1000";
		
		try {
			// Determine vocabulary
			Set<String> vocabulary = determineVocabulary(trainData, 1);
			
			// Build a bigram language model from given training data
			System.out.println("Building a bigram language model... ");
			BigramModel lm = new BigramModel(trainData, vocabulary);

			// Print the probability that the bigram language model assigns
			// to this test sentence.
			String test ="Man muß diese Abhängigkeiten schaffen .";
			double p = lm.score(test);
			
			System.out.println();
			System.out.println("One test sentence:");
			System.out.println(test + "  " + p);
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

	}

}
