package de.hhu.phil.smt.lm;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Scanner;
import java.util.Set;

import de.hhu.phil.smt.SMT;

/*
 * Author: kaeshammer@phil.uni-duesseldorf.de
 * 
 * Class für Uebung 3 of the course Statistische Maschinelle Übersetzung 2013 
 * at HHU
 */
public class Uebung3 {
	
	/*
	 * @Param dataFile file with the training data
	 * @Param threshold frequency threshold
	 * Determines the vocabulary given the training data. 
	 * Only words that occur more frequently than the threshold are
	 * part of the vocabulary.
	 */
	private static Set<String> determineVocabulary(String dataFile, int threshold) throws FileNotFoundException {
		// data structure to count word frequencies
		HashMap<String, Integer> counts = new HashMap<String, Integer>();
		// read in the training data file line by line
		Scanner input = new Scanner(new File(dataFile));
		String line = "";
		while (input.hasNextLine()) {
			line = input.nextLine();
			// tokenize the sentence
			String[] tokens = SMT.tokenizeWS(line);
			// fill the data structure
			for (String tok : tokens) {
				Integer c = counts.get(tok);
				if (c == null) {
					c = 0;
				}
				counts.put(tok, c+1);
			}
		}
		input.close();
		
		// add words which we have seen more often than threshold to the vocabulary
		Set<String> vocabulary = new HashSet<String>();
		for (Map.Entry<String, Integer> entry : counts.entrySet()) {
			if (entry.getValue() > threshold) {
				vocabulary.add(entry.getKey());
			}
		}
		
		vocabulary.add(SMT.START_TOK);
		vocabulary.add(SMT.END_TOK);
		vocabulary.add(SMT.UNK);
		
		return vocabulary;
	}

	/**
	 * @param args
	 */
	public static void main(String[] args) {
		
		String trainData = "data/train.de.tok.50.h1000";
		//String trainData = "data/train.de.tok.50.h500000";
		
		try {
			// Determine vocabulary
			System.out.println("Building the vocabulary... ");
			Set<String> vocabulary = determineVocabulary(trainData, 1);
			
			// Build a bigram language model from given training data
			System.out.println("Building a bigram language model... ");
			BigramModel lm2 = new BigramModel(trainData, vocabulary);
			
			// Print the probability that the language model assigns
			// to this test sentence.
			String test ="Man muß diese Abhängigkeiten schaffen .";
			double p2 = lm2.score(test);
			
			System.out.println();
			System.out.println("One test sentence: " + test);
			System.out.println("BigramLM " + p2);
			
			// Test file
			String testData = "data/train.de.tok.50.t50";
			System.out.println();
			System.out.println("Test file: " + testData);
				
			double p2File = lm2.scoreFile(testData);
			System.out.println("BigramLM probability " + p2File);
			
			// log probability 
			double p2FileLog = lm2.logScoreFile(testData);
			System.out.println("BigramLM log-probability " + p2FileLog);
			System.out.println();
						
			
			// Build a trigram language model from given training data
			System.out.println("Building a trigram language model... ");
			TrigramModel lm3 = new TrigramModel(trainData, vocabulary);

			// Try trigram LM on test sentence
			double p3 = lm3.score(test);
			System.out.println();
			System.out.println("One test sentence: " + test);
			System.out.println("TrigramLM " + p3);
			
			
			// Try trigram LM on test file
			double p3FileLog = lm3.logScoreFile(testData);
			System.out.println();
			System.out.println("Test file: " + testData);
			System.out.println("TrigramLM log-probability " + p3FileLog);
			System.out.println();

			
			// Perplexity
			double perp2 = SMT.perplexity(testData, lm2);
			double perp3 = SMT.perplexity(testData, lm3);
			System.out.println("Perplexity Bi " + perp2);
			System.out.println("Perplexity Tri " + perp3);
			
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		}

	}

}
