package de.hhu.phil.smt.misc;

/*
 * Author: kaeshammer@phil.uni-duesseldorf.de
 * 
 * This class represents a pair where the first element is of class A 
 * and the second element is of class B. 
 * It implements the interface Comparable and a hash function. It should 
 * thus be possible to use the Pair class in collections that rely on the
 * order of elements (e.g. TreeSet) and/or the hashcode (e.g. HashMap). * 
 */

public class Pair<A extends Comparable<? super A>, B extends Comparable<? super B>> 
								implements Comparable<Pair<A, B>> {
	
	private final A first;
	private final B second;

	public Pair(A a, B b) {
		first = a;
		second = b;
	}
	  
	public A getFirst() {
		return first;
	}
	  
	public B getSecond() {
		return second;
	}
	
	public String toString() {
		StringBuffer buffer = new StringBuffer();
		buffer.append("[");
		buffer.append(first);
		buffer.append(", ");
		buffer.append(second);
		buffer.append("]");
		return buffer.toString();
	}
	
	/*
	 * @Param p is a non-null pair
	 * @Throws NullPointerException if p is null
	 * Establishes an order between pairs. Returns 0 if p
	 * and this are equal. If the first elements of p and
	 * this are equal, then the order is determined by the second
	 * element, and vice versa. Otherwise the order is determined
	 * by the first element.
	 */
	@Override
	public int compareTo(Pair<A, B> p) { 
		if (p.equals(this)) { 
			return 0;
		}
		else {
			int f = first.compareTo(p.getFirst());
			int s = second.compareTo(p.getSecond());
			if (f == 0) {
				return s;
			}
			else if (s == 0) {
				return f;
			}
			else {
				return f;
			}
		}
	}
	
	/*
	 * @Param o is an Object
	 * o and this are equal if o is a Pair and their first and
	 * second elements are equal.
	 */
	@Override
	public boolean equals(Object o) {
		if (o instanceof Pair) { 
			Pair<?, ?> p = (Pair<?, ?>) o;
			if (p.getFirst().equals(first) && p.getSecond().equals(second)) {
				return true;
			}
		}
		return false;
	}
	
	public int hashCode() { 
		return first.hashCode() + (31 * second.hashCode());
	}
}
