package de.hhu.phil.smt.misc;


import java.io.File;
import java.io.FileNotFoundException;
import java.util.HashSet;
import java.util.Scanner;
import java.util.Set;

import de.hhu.phil.smt.SMT;

/*
 * Class that represents the translation co-occurrences required for
 * Uebung 1 of the SMT class.
 * 
 * Proposed e.g. by Eser Demirci
 */
public class TransCooc {
	
	private Set<Pair<String, String>> pairsA = new HashSet<>();
	private Set<Pair<Pair<String, String>, String>> pairsB = new HashSet<>();

	/*
	 * Constructor: Reads data from fileForeign and fileEnglish. The files
	 * must contain sentence aligned text, one sentence per line.
	 */
	public TransCooc(String fileForeign, String fileEnglish) throws FileNotFoundException {

		Scanner inputF = new Scanner(new File(fileForeign));
		Scanner inputE = new Scanner(new File(fileEnglish));
		String foreign = "";
		String english = "";
		int c = 0;
		while (inputF.hasNextLine() && inputE.hasNextLine()) {
			foreign = inputF.nextLine();
			english = inputE.nextLine();
			c++;
			//System.out.println(foreign);
			//System.out.println(english);
			
			addToCooc(foreign,english);
		}
		
		inputF.close();
		inputE.close();
		System.out.println("Number of read sentence pairs: " + c);
	}

	/*
	 * Fill the co-occurrence data structure
	 */
	private void addToCooc(String foreign, String english) {
		
		// Tokenize the sentence into words
		String[] toksF = SMT.tokenizeWS(foreign);
		String[] toksE = SMT.tokenizeWS(english);
		
		// (a)
		for (String tokenF : toksF) {
			for (String tokenE : toksE) {
				pairsA.add(new Pair<String, String>(tokenF, tokenE));
			}
		}

		// (b)
		for (int tokenF2 = 0; tokenF2 < toksF.length; tokenF2++) {
			for (int tokenF1 = 0; tokenF1 < tokenF2; tokenF1++) {
				for (String tokenE : toksE) {
					pairsB.add(new Pair<Pair<String, String>, String>(
							new Pair<String, String>(toksF[tokenF1], toksF[tokenF2]),
							tokenE));
				}
			}
		}
	}

	/*
	 * Print the number of translation co-occurrences that were found
	 */
	public void printSize() {
		System.out.println("Number of wordpairs (1-1 relation): "
				+ pairsA.size());

		System.out.println("Number of wordpairs (2-1 relation): "
				+ pairsB.size());
	}

}
